/*-
 *  Copyright (c) 2001  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

#include <errno.h>
#include <stdio.h>
#include <string.h>

#include "vpass.h"
#include "vp_err.h"

static char	cvs_id[] __unused =
"$Ringlet: vp_err.c 2670 2008-10-09 15:51:23Z roam $";

FILE		*vp_errfp;

static const char *errs[VP_ERR_LAST] = {
	"no error",
	"out of memory",
	"bad command-line arguments",
	"fgets",
	"network subsystem not initialized",
	"no UCSPI protocol specified",
	"network read error",
	"network write error",
	"communications protocol error",
	"INTERNAL ERROR"
};

static int errsn[VP_ERR_LAST] = {
	0, 0, 0,	/* NONE, NOMEM, CMDLINE */
	1,		/* FGETS */
	0,		/* NET_NOTINIT */
	0,		/* NET_UCS_NOPROTO */
	1, 1,		/* READ, WRITE */
	0,		/* COMM */
	0		/* INT */
};

static char	errbuf[512];

/*
 * Function:
 * 	vp_strerror			- build an error message string
 * Inputs:
 * 	code				- error code
 * Returns:
 * 	pointer to static error message string
 * Modifies:
 * 	errbuf
 */

char *
vp_strerror(vp_err_t code) {
	
	if (code > VP_ERR_INT)
		code = VP_ERR_INT;

	if (errsn[code])
		snprintf(errbuf, sizeof(errbuf), "%s: %s", errs[code],
		    strerror(errno));
	else
		snprintf(errbuf, sizeof(errbuf), "%s", errs[code]);
	
	return (errbuf);
}

/*
 * Function:
 * 	vp_prerror			- print error message and context
 * Inputs:
 * 	s				- context string to be printed
 * 	code				- error code
 * Returns:
 * 	code
 * Modifies:
 * 	nothing; writes to stderr
 */

vp_err_t
vp_prerror(const char *s, vp_err_t code) {
	
	if (code > VP_ERR_INT)
		code = VP_ERR_INT;
	
	if (vp_errfp == NULL)
		vp_errfp = stderr;
	fprintf(vp_errfp, "vpass error: %s: %s\n", s, vp_strerror(code));
	return (code);
}
