/*-
 *  Copyright (c) 2001, 2008  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "vpass.h"
#include "vp_err.h"
#include "vp_net.h"
#include "vp_net_ucspi.h"
#include "vp_util.h"

static char	cvs_id[] __unused =
"$Ringlet: vp_net.c 2673 2008-10-09 15:59:40Z roam $";

vp_nmode	vp_net_mode;

static vp_err_t	vp_net_sendexp(int fds[2], const char *s, char **resp,
		int *perr);
static vp_err_t	vp_net_cmdresp2(int fds[2], const char *cmd, const char *arg);

vp_err_t
vp_net_connect(int fds[2]) {

	switch (vp_net_mode) {
		case VP_NET_UCSPI:
			return (vp_net_ucspi_connect(fds));

		default:
			return (VP_ERR_NET_NOTINIT);
	}
}

vp_err_t
vp_net_close(int fds[2]) {

	close(fds[0]);
	close(fds[1]);
	return (VP_ERR_NONE);
}

vp_err_t
vp_net_talk(int fds[2]) {
	char *line;
	size_t sz;
	vp_err_t r;

	/* Handshake: expect the server to tell us who they are */
	if (r = vp_readline(fds[0], &line, &sz), r)
		return (r);
	if (line == NULL)
		return (VP_ERR_INT);
	RDBG(("Remote server greeting: %s", line));
	/* FIXME: examine that to determine POP3 or SMTP responses */
	free(line);

	if (r = vp_net_cmdresp2(fds, "USER", vp_user), r)
		return (r);
	if (r = vp_net_cmdresp2(fds, "PASS", vp_oldp), r)
		return (r);

	if (vp_create) {
		if (r = vp_net_cmdresp2(fds, "NEWUSER", vp_cuser), r)
			return (r);
	} else if (vp_cuser != NULL) {
		if (r = vp_net_cmdresp2(fds, "CUSER", vp_cuser), r)
			return (r);
	}

	if (r = vp_net_cmdresp2(fds, "NEWPASS", vp_newp), r)
		return (r);

	return (VP_ERR_NONE);
}

static vp_err_t
vp_net_cmdresp2(int fds[2], const char *cmd, const char *arg) {
	char *cmdstr, *resp;
	int err;
	vp_err_t r;

	cmdstr = resp = NULL;
	do {
		if (r = vp_strmcat(&cmdstr, 3, cmd, " ", arg), r)
			break;
		RDBG(("sending command '%s'\n", cmdstr));
		if (r = vp_net_sendexp(fds, cmdstr, &resp, &err), r)
			break;
		RDBG(("received response '%s', err %d\n", resp, err));
		if (err) {
			fprintf(vp_errfp, "Sending '%s': %s\n", cmd, resp);
			r = VP_ERR_COMM;
		}
	} while (0);

	free(cmdstr);
	free(resp);
	return (r);
}

static vp_err_t
vp_net_sendexp(int fds[2], const char *s, char **resp, int *perr) {
	char *line;
	size_t sz;
	int err;
	vp_err_t r;

	if (r = vp_writeline(fds[1], s, 1), r)
		return (r);
	if (r = vp_readline(fds[0], &line, &sz), r)
		return (r);
	/* FIXME: fugly and incorrect for an SMTP-style dialogue */
	err = !!memcmp(line, "+OK", 3);

	*resp = line;
	*perr = err;
	return (VP_ERR_NONE);
}
