/*-
 *  Copyright (c) 2001  Peter Pentchev
 *  All rights reserved.
 * 
 *  Redistribution and use in source and binary forms, with or without
 *  modification, are permitted provided that the following conditions
 *  are met:
 *  1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 
 *  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 *  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 *  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 *  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 *  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 *  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 *  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 *  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *  SUCH DAMAGE.
 */

#include <stdlib.h>

#include "vpass.h"
#include "vp_net.h"
#include "vp_net_ucspi.h"
#include "vp_util.h"

#ifdef EBUG
#include <stdio.h>
#endif

static char	cvs_id[] __unused =
"$Ringlet: vp_net_ucspi.c 2670 2008-10-09 15:51:23Z roam $";

static const char	*ucs_proto, *ucs_remaddr, *ucs_remport;

vp_err_t
vp_net_ucspi_connect(int fds[2]) {
	char *name, *v;
	vp_err_t r;

	RDBG(("ucspi: examining the environment\n"));
	if (ucs_proto = getenv("PROTO"), ucs_proto == NULL)
		return (VP_ERR_NET_UCS_NOPROTO);

	if (r = vp_strmcat(&name, 2, ucs_proto, "REMOTEIP"), r)
		return (r);
	if (v = getenv(name), v != NULL) {
		RDBG(("ucspi: got remote IP address %s\n", v));
		ucs_remaddr = v;
	}
	free(name);

	if (r = vp_strmcat(&name, 2, ucs_proto, "REMOTEPORT"), r)
		return (r);
	if (v = getenv(name), v != NULL) {
		RDBG(("ucspi: got remote IP port %s\n", v));
		ucs_remport = v;
	}
	free(name);

	fds[0] = 6;
	fds[1] = 7;
	return (VP_ERR_NONE);
}
